%% To run 4-fold cross validated leave-region-out GPFA model for Power 264 parcellation

addpath(genpath('gpfa_v0203')); %GPFA toolbox for function gpfaEngine

%%
clear;

%%%%%%%%%%%%%% changeable params %%%%%%%%%%%%%%%%%%%%
task_names_run  = {'rest',   'WM',    'lang',  'motor', 'social', 'gambling', 'relational', 'emotion'};
binWidth = 1;
startTau = 10;
kernSD = 30;
segLength = 20;
emMaxIters = 100000;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% fMRI preprocessed series - power parcellated
% The subject IDs of these 100 subjects used for template are in:
% Fig1/data/template_subjects_IDs.mat
all_mat_name = '/core_scripts/data/tc_all_lr_power_808_subs.mat'; 

display_name_cells={'rest',   'WM',    'lang',  'motor', 'social', 'gambling', 'relational', 'emotion'}; %human name for the task variables
task_names_cells = {'resting','task7', 'task2', 'task4', 'task6',  'task3',    'task5',   'task1'}; %matrix names

folder = sprintf('/core_scripts/data/cv_power_gpfa_data');
if ~isdir(folder)
    mkdir(folder);
end


num_folds = 4;
fold_size = 25;

for i = 1:8
    for xDim = [5 10 20 30 40 50 60 80 100]
        
        for k=1:4
            i
            xDim
            k
            train_indices_kth = [];
            
            for t = 1:num_folds
                if t==k
                    continue;
                end
                for u = (t-1)*fold_size+1:t*fold_size
                    train_indices_kth = [train_indices_kth 500+u]; % 75 subs
                end
            end
            
            test_indices_kth = [];
            
            
            for u = (k-1)*fold_size+1:k*fold_size
                test_indices_kth = [test_indices_kth 500+u]; %25 subs
            end
            
            
            task_mat_name = sprintf('tc_%s_concat', task_names_cells{strcmp(display_name_cells, task_names_run{i})});
            
            load(all_mat_name,task_mat_name);
            
            
            data_all = eval(task_mat_name);
            [T, no_regions, no_subs] = size(data_all);
            disp('train data');
            c = 1;
            for sub = train_indices_kth
                disp(sub);
                lim = min(size(data_all,1)-10, 100);
                current = data_all(11:lim+10,:,sub);
                current = zscore(current);
                seqTrain(c).trialId = num2str(sub);
                seqTrain(c).y = current';
                seqTrain(c).T = size(current,1);
                c = c+1;
            end
            disp('test data');
            c = 1;
            for sub = test_indices_kth
                disp(sub);
                lim = min(size(data_all,1)-10, 100);
                current = data_all(11:lim+10,:,sub);
                current = zscore(current);
                seqTest(c).trialId = num2str(sub);
                seqTest(c).y = current';
                seqTest(c).T = size(current,1);
                c = c+1;
            end
            
            
            
            clear(task_mat_name, 'data_all');
            
            fname = sprintf('%s/gpfa_%s_power_xDim_%03d_cv_%d.mat', folder, task_names_run{i}, xDim, k);
            
            extraOpts = {'segLength'    segLength    'kernSDList'    kernSD    'emMaxIters'    emMaxIters    'startTau'    startTau};
            
            gpfaEngine(seqTrain, seqTest, fname, 'xDim', xDim, 'binWidth', binWidth, extraOpts{:});
        end
    end
end


%% get error for all #dims
% to calculate total error at different xDims from the cross-validated output data

clear;
addpath(genpath('gpfa_v0203'));

task_names_run  = {'rest',   'WM',    'lang',  'motor', 'social', 'gambling', 'relational', 'emotion'};


folder = sprintf('/core_scripts/data/cv_power_gpfa_data');
xDim_all = [5 10 20 30 40 50 60 80 100];
num_folds = 4;
for tsk_idx = 1:8
    tsk_idx
    
    %calculating error without orthogonalisation at diiferent xDims
    err_all_no_orth = [];
    for xdim_T = xDim_all
      err = 0;
      err_denom = 0;
      total_points = 0;
      for k=1:num_folds
        fname = sprintf('%s/gpfa_%s_power_xDim_%03d_cv_%d.mat', folder, task_names_run{tsk_idx}, xdim_T, k);
            
        load(fname);
        fn              = sprintf('ycsOrth%02d', xDim);
        Ycs             = [seqTest.(fn)];
        YtestRaw       = [seqTest.y];
        
        err = err + sum((Ycs(:) - YtestRaw(:)).^2); 
        total_points = total_points + length(Ycs(:));
      end
        err_all_no_orth = [err_all_no_orth err/total_points];
    end
    
    xDims_no_orth = xDim_all;
    
    %calculating error with orthogonalisation at diferent xDims
    xDims_orth = 5:xDim;
    err_all_orth = zeros(1, length(xDims_orth));
    for k=1:num_folds
        fname = sprintf('%s/gpfa_%s_power_xDim_%03d_cv_%d.mat', folder, task_names_run{tsk_idx}, xDim, k);
            
        load(fname);
        c = 0;
        for xdim_T = xDims_orth

            fn              = sprintf('ycsOrth%02d', xdim_T);
            Ycs             = [seqTest.(fn)];
            YtestRaw       = [seqTest.y];
            err = sum((Ycs(:) - YtestRaw(:)).^2); 
            c = c+1;
            err_all_orth(c) = err_all_orth(c) + err;
        end
    end
    err_all_orth = err_all_orth/total_points;
    save(sprintf('data/error_data_cv_power_%s_%03d_normalized_mean.mat', task_names_run{tsk_idx}, xdim_T), 'xDims_no_orth', 'err_all_no_orth', 'xDims_orth', 'err_all_orth');
end


%% get optimal #dim across tasks

clear;
addpath(genpath('../../addons/gpfa_v0203'));

task_names_run  = {'rest',   'WM',    'lang',  'motor', 'social', 'gambling', 'relational', 'emotion'};

colors = distinguishable_colors(8);

xdim_T = 100;
min_xDim_all = [];
f_all = [];
xDim_all = 5:xdim_T;
for tsk_idx = [1 2 3 4 5 6 7 8]
    load(sprintf('data/error_data_cv_power_%s_%03d_normalized_mean.mat', task_names_run{tsk_idx}, xdim_T));
    f=fit(xDims_no_orth', err_all_no_orth', 'exp2');
   
    [min_err, idx] = min(f(xDim_all));
    
    f_all{tsk_idx} = f;
    min_xDim_all = [min_xDim_all xDim_all(idx)];
end

dev_err_all = [];

for xDim = xDim_all
    dev_err = 0;
    
    for tsk_idx = [1 2 3 4 5 6 7 8]
        f_curr = f_all{tsk_idx};
        err_curr = f_curr(xDim) - f_curr(min_xDim_all(tsk_idx));
        dev_err = dev_err + err_curr;
    end
    dev_err_all = [dev_err_all dev_err];
end


[~, opt_xDim_idx] = min(dev_err_all);

opt_xDim = xDim_all(opt_xDim_idx);
fprintf('Optimal Dimension : %d\n',opt_xDim); %42

%% to plot latent dimension vs error

clear;
close all;
task_names_run  = {'rest',   'WM',    'lang',  'motor', 'social', 'gambling', 'relational', 'emotion'};

% colors = distinguishable_colors(8); 
colors = [0,0,1;1,0,0;0,1,0;0,0,0.172413793103448;1,0.103448275862069,0.724137931034483;1,0.827586206896552,0;0,0.344827586206897,0;0.517241379310345,0.517241379310345,1];

close all;
figure('visible', 'on', 'position' , [0 0 0.8*600 0.8*450]);
xdim_T = 100;
h_all = [];
c = 0;
for tsk_idx = [1 2 3 4 5 6 7 8]
    tsk_idx
    c = c+1;
    load(sprintf('data/error_data_cv_power_%s_%03d_normalized_mean.mat', task_names_run{tsk_idx}, xdim_T));
    f=fit(xDims_no_orth', err_all_no_orth', 'exp2');
    h_all = [h_all plot(5:xdim_T, f(5:xdim_T), 'lineWidth', 1.5, 'color', colors(tsk_idx, :))];
    hold on;
    scatter(xDims_no_orth,err_all_no_orth,25,'o','filled','MarkerEdgeColor',colors(tsk_idx, :),...
              'MarkerFaceColor',colors(tsk_idx, :));
    hold on;
    [min_err, idx] = min(f(5:xdim_T));
    temp = 5:xdim_T;
    
    scatter(temp(idx),min_err,55,'^','filled',...%'MarkerEdgeColor',colors(tsk_idx, :),...
              'MarkerFaceColor',colors(tsk_idx, :), 'LineWidth', 1.5);
    hold on;
      
    legend_labels{c} = task_names_run{tsk_idx};
    
end
hleg = legend(h_all, legend_labels);
% title(hleg,'task');
xlabel('latent dimension');
ylabel('mean squared error');
hold on;
plot([42 42],[0.5 0.65],'--k');

