addpath(genpath('../../../addons/gpfa_v0203'));

%%
clear;

%%%%%%%%%%%%%% changeable params %%%%%%%%%%%%%%%%%%%%
task_names_run  = {'rest',   'WM',    'lang',  'motor', 'social', 'gambling', 'relational', 'emotion'};
binWidth = 1;
startTau = 10;
kernSD = 30;
segLength = 20;
emMaxIters = 100000;
xDim = 42
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


all_mat_name = '/core_scripts/data/tc_all_lr_power_808_subs.mat';% fMRI preprocessed series - power parcellated

display_name_cells={'rest',   'WM',    'lang',  'motor', 'social', 'gambling', 'relational', 'emotion'}; %human name for the task variables
task_names_cells = {'resting','task7', 'task2', 'task4', 'task6',  'task3',    'task5',      'task1'}; %matrix names

folder = sprintf('/core_scripts/data/power_templates_as_trials');

if ~isdir(folder)
        mkdir(folder);
end

for i = 1:8
    task_name = task_names_run{i}

    task_mat_name = sprintf('tc_%s_concat', task_names_cells{strcmp(display_name_cells, task_name)})
       
    load(all_mat_name,task_mat_name);
   
    data_all = eval(task_mat_name);
    [T, no_regions, no_subs] = size(data_all);
    
    c = 0;
    for sub = 501:600
        c = c+1;
        fprintf('%d ', sub);
        lim = min(size(data_all,1)-10, 100);
        current = data_all(11:lim+10,:,sub);
        current = zscore(current);
        seqTrain(c).trialId = num2str(sub);
        seqTrain(c).y = current';
        seqTrain(c).T = size(current,1);
    end
    fprintf('\n');
    fname = sprintf('%s/gpfa_%s_power_xDim_%02d.mat', folder, task_names_run{i}, xDim);
    
    extraOpts = {'segLength'    segLength    'kernSDList'    kernSD    'emMaxIters'    emMaxIters    'startTau'    startTau};
    
    gpfaEngine(seqTrain, [], fname, 'xDim', xDim, 'binWidth', binWidth, extraOpts{:});
    
end

%% get template series
addpath(genpath('gpfa_v0203'));
addpath(genpath('chronux_2_12'));
%% GPFA

clear;
close all;

task_names_run = {'rest',   'WM',    'lang',  'motor', 'social', 'gambling', 'relational', 'emotion'};
folder = sprintf('/core_scripts/data/power_templates_as_trials');

for tsk_idx = 1:length(task_names_run)
    fname = sprintf('%s/gpfa_%s_power_xDim_42.mat', folder, task_names_run{tsk_idx});
    load(fname, 'seqTrain', 'estParams');
    mean_trajectories = [];
    for component_index = 1:length(estParams.gamma)
        time_series_all_org = [];
        for i=1:length(seqTrain)
            time_series_all_org=[time_series_all_org; seqTrain(i).xsm(component_index, :)];
        end
        mean_trajectories = [mean_trajectories; mean(time_series_all_org)];
    end
    
    template_data(tsk_idx).mean_trajectories = mean_trajectories;
    template_data(tsk_idx).estParams = estParams;
    template_data(tsk_idx).task_name = task_names_run{tsk_idx};
end
save(sprintf('data/template_data_power_templates.mat'), 'template_data');

%% orthogonalised GPFA

clear;
close all;

%%%%%%%%%%%%%% changeable params %%%%%%%%%%%%%%%%%%%%
task_names_run = {'rest',   'WM',    'lang',  'motor', 'social', 'gambling', 'relational', 'emotion'};
folder = sprintf('/core_scripts/data/power_templates_as_trials');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


for tsk_idx = 1:length(task_names_run)
    fname = sprintf('%s/gpfa_%s_power_xDim_42.mat', folder, task_names_run{tsk_idx});
    load(fname, 'seqTrain', 'estParams');
    C               = estParams.C;
    X               = [seqTrain.xsm];    
    [Xorth, Corth]  = orthogonalize(X, C); 
    seqTrain        = segmentByTrial(seqTrain, Xorth, 'xorth');
    estParams.Corth = Corth;
    
    mean_trajectories = [];
    for component_index = 1:size(Corth, 2)
        time_series_all_org = [];
        for i=1:length(seqTrain)
            time_series_all_org=[time_series_all_org; seqTrain(i).xorth(component_index, :)];
        end
        mean_trajectories = [mean_trajectories; mean(time_series_all_org)];
    end
    
    template_data(tsk_idx).mean_trajectories_orth = mean_trajectories;
    template_data(tsk_idx).estParams_with_orth = estParams;
    template_data(tsk_idx).task_name = task_names_run{tsk_idx};
end
save(sprintf('data/template_data_power_templates_with_orth.mat'), 'template_data');

clear;
close all;
%% PCA : taking no. of components comparable to gpfa: 42
numComp=42;
%%
%%%%%%%%%%%% changeable params %%%%%%%%%%%%%%%%%%%%
task_names_run = {'rest',   'WM',    'lang',  'motor', 'social', 'gambling', 'relational', 'emotion'};
task_names_cells = {'resting','task7', 'task2', 'task4', 'task6',  'task3',    'task5',      'task1'};
display_name_cells={'rest',   'WM',    'lang',  'motor', 'social', 'gambling', 'relational', 'emotion'}; %human name for the task variables
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

all_mat_name = '/core_scripts/data/tc_all_lr_power_808_subs.mat';
for tsk_idx = 1:8
    task_mat_name = sprintf('tc_%s_concat', task_names_cells{strcmp(display_name_cells, task_names_run{tsk_idx})});
    
    load(all_mat_name,task_mat_name);
    
    
    data_all = eval(task_mat_name);
    clear(task_mat_name);
    [T, no_regions, no_subs] = size(data_all);
    
    data = [];
    
    for sub = 501:1:600
        disp(sub);
        lim = min(size(data_all,1)-10, 100);
        current = data_all(11:lim+10,:,sub);
        current = zscore(current);
        data = [data; current];
    end
    
    
    
    [COEFF, SCORE] = pca(data);
    
    estParams.coeffs = COEFF(:, 1:numComp);
    
    mean_trajectories = [];
    parfor component_index = 1:size(estParams.coeffs, 2)
        time_series_all_org = [];
        for i=1:100
            time_series_all_org=[time_series_all_org; SCORE((i-1)*100+1:i*100, component_index)'];
        end
        mean_trajectories = [mean_trajectories; mean(time_series_all_org)];
    end
    
    template_data(tsk_idx).mean_trajectories_PCA = mean_trajectories;
    template_data(tsk_idx).estParams = estParams;
    template_data(tsk_idx).task_name = task_names_run{tsk_idx};
end
save(sprintf('data/template_data_power_templates_PCA.mat'), 'template_data');
