%% project all available test scans using all tasks' template parameters - for all timepoints

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Subjects' task scans were projected for same number of timepoints across tasks, as well as all available timepoints :
% 
% (1) Sha_trajs_using_all_<task_i>.mat
%     projecting test subjects' scans of task_i - initial 100 timepoints (from 11:110) - onto reduced dimension of each task
% 
% (2) Sha_entiretrajs_using_all_<task_i>.mat
%     projecting test subjects' scans of task_i - all available timepoints (entire series) - onto reduced dimension of each task
% 
% (3) Sha_extracted_trajs_<task_i>_entire.mat    
%     projecting test subjects' scans of task_i - all available timepoints (entire series) - onto reduced dimension of task_i
%     This is just a subset of (2) saved as a separate dataset.
% 
% (4) Sha_extracted_trajs_<task_i>_entire_templates.mat    
%     projecting template subjects' scans of task_i - all available timepoints (entire series) - onto reduced dimension of task_i
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear;
clc;
close all;

addpath(genpath('gpfa_v0203'));
%% GPFA
task_names_run = {'rest',   'WM',    'lang',  'motor', 'social', 'gambling', 'relational', 'emotion'};
display_name_cells={'rest',   'WM',    'lang',  'motor', 'social', 'gambling', 'relational', 'emotion'}; %human name for the task variables
task_names_cells = {'rest1','task7', 'task2', 'task4', 'task6',  'task3',    'task5',      'task1'}; %matrix names

% fMRI preprocessed series - power parcellated
% The subject IDs of these 900 subjects used as test dataset are in:
% Fig1/data/predictOn_subjects_IDs.mat

all_mat_name = '/core_scripts/data/tc_all_LR_power_predictOn900.mat';

for i = 1:8
    
    task_name_actual = task_names_run{i}
    task_mat_name = sprintf('tc_%s_concat', task_names_cells{strcmp(display_name_cells, task_name_actual)});
    
    load(all_mat_name,task_mat_name);
    
    data_all = eval(task_mat_name);
    [T, no_regions, no_subs] = size(data_all);
    
    c = 0;
    for sub = 1:900
        c = c+1;
        fprintf('%d ', sub);
        current = data_all(:,:,sub);
        current = zscore(current);
        seqTrainRem(c).trialId = num2str(sub);
        seqTrainRem(c).y = current';
        seqTrainRem(c).T = size(current,1);
    end
    
    trajs_all=[];
    for j=1:8
        task_name_using = task_names_run{j}
        load(sprintf('/core_scripts/data/power_templates_as_trials/gpfa_%s_power_xDim_42.mat', task_name_using), 'estParams');
        [blah, ll] = exactInferenceWithLL(seqTrainRem, estParams);
        trajs= [];
        parfor c=1:length(blah)
            trajs(c).T = blah(c).T;
            trajs(c).y = blah(c).y;
            trajs(c).trialId = blah(c).trialId;
            trajs(c).xsm = blah(c).xsm;
        end
        trajs_all(j).trajs = trajs;
        trajs_all(j).task_name_using = task_name_using;
        
    end
    
   save(sprintf('/core_scripts/data/Sha_pairedProjections_power/Sha_entiretrajs_using_all_%s.mat', task_name_actual), 'trajs_all','-v7.3');

end

%% orthogonalised GPFA
clear;
close all;

%%%%%%%%%%%%%% changeable params %%%%%%%%%%%%%%%%%%%%
task_names_run = {'rest',   'WM',    'lang',  'motor', 'social', 'gambling', 'relational', 'emotion'};
display_name_cells={'rest',   'WM',    'lang',  'motor', 'social', 'gambling', 'relational', 'emotion'}; %human name for the task variables
task_names_cells = {'rest1','task7', 'task2', 'task4', 'task6',  'task3',    'task5',      'task1'}; %matrix names
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

all_mat_name = '/core_scripts/data/tc_all_LR_power_predictOn900.mat';

for i = 1:8
    
    task_name_actual = task_names_run{i}
    
    task_mat_name = sprintf('tc_%s_concat', task_names_cells{strcmp(display_name_cells, task_name_actual)})
    
    load(all_mat_name,task_mat_name);
    
    data_all = eval(task_mat_name);
    [T, no_regions, no_subs] = size(data_all);
    
    c = 0;
    for sub = 1:900
        c = c+1;
        fprintf('%d ', sub);
        current = data_all(:,:,sub);
        current = zscore(current);
        seqTrainRem(c).trialId = num2str(sub);
        seqTrainRem(c).y = current';
        seqTrainRem(c).T = size(current,1);
    end
    clear trajs_all;
    
    for j=1:8
        task_name_using = task_names_run{j}
        load(sprintf('/core_scripts/data/power_templates_as_trials/gpfa_%s_power_xDim_42.mat', task_name_using), 'estParams');
        [blah, ll] = exactInferenceWithLL(seqTrainRem, estParams);
        
        C               = estParams.C;
        X               = [blah.xsm];    
        [Xorth, Corth]  = orthogonalize(X, C); 
        blah        = segmentByTrial(blah, Xorth, 'xorth');
        trajs= [];
        for c=1:length(blah)
%             trajs(c).T = blah(c).T;
            %trajs(c).y = blah(c).y;
            trajs(c).trialId = blah(c).trialId;
            %trajs(c).xsm = blah(c).xsm;
            trajs(c).xorth = blah(c).xorth;
        end
        trajs_all(j).trajs = trajs;
        trajs_all(j).task_name_using = task_name_using;
    end
        save(sprintf('/core_scripts/data/Sha_pairedProjections_power/Sha_entiretrajs_using_all_%s_orth.mat', task_name_actual), 'trajs_all');

end

%% PCA

clear;
close all;

%%%%%%%%%%%%%% changeable params %%%%%%%%%%%%%%%%%%%%
task_names_run = {'rest',   'WM',    'lang',  'motor', 'social', 'gambling', 'relational', 'emotion'};
display_name_cells={'rest',   'WM',    'lang',  'motor', 'social', 'gambling', 'relational', 'emotion'}; %human name for the task variables
task_names_cells = {'rest1','task7', 'task2', 'task4', 'task6',  'task3',    'task5',      'task1'}; %matrix names
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

all_mat_name = '/core_scripts/data/tc_all_LR_power_predictOn900.mat';

load('/core_scripts/data/template_data_power_templates_PCA.mat');

for i = 1:8
    
    task_name_actual = task_names_run{i};fprintf('\nactual: %s using...',task_name_actual);
    task_mat_name = sprintf('tc_%s_concat', task_names_cells{strcmp(display_name_cells, task_name_actual)});
    load(all_mat_name,task_mat_name);
    
    data_all = eval(task_mat_name);
    clear(task_mat_name);
    [T, no_regions, no_subs] = size(data_all); 
    
        clear trajs_all;

    for j=1:8
        task_name_using = task_names_run{j};disp(task_name_using);
        clear trajs;
        parfor sub=1:900
            current = data_all(:,:,sub);
            current = zscore(current);
            trajs(sub).xsm = current*template_data(j).estParams.coeffs;
            trajs(sub).T = size(current,1);
            trajs(sub).trialId = num2str(sub);
        end
        
        trajs_all(j).trajs = trajs;
        trajs_all(j).task_name_using = task_name_using;
        
    end
            save(sprintf('/core_scripts/data/Sha_pairedProjections_power/Sha_entiretrajs_using_all_%s_pca.mat', task_name_actual), 'trajs_all','-v7.3');

end




