addpath(genpath('../../../addons/gpfa_v0203'));
addpath(genpath('../../../addons/chronux_2_12'));
%%
% The following classification is being done with 8 labels in training and testing : the 7 tasks and resting state.
% Same was done with just 7 tasks, excluding all resting state data, and accuracies are saved in
% Fig2/data/GPFA42_confusion_mat_without_rest_label

clc;
clear;
close all;
%%%%%%%%%%%%%% changeable params %%%%%%%%%%%%%%%%%%%%
task_names_run  = {'rest',   'WM',    'lang',  'motor', 'social', 'gambling', 'relational', 'emotion'};
display_name_cells={'rest',   'WM',    'lang',  'motor', 'social', 'gambling', 'relational', 'emotion'}; %human name for the task variables
task_names_cells = {'rest1','task7', 'task2', 'task4', 'task6',  'task3',    'task5',      'task1'}; %matrix names

load('/Fig1/data/template_data_power_templates.mat');

for set_size = [1]
    set_size
    confusion_mat = zeros(8, 8);
    for tsk_idx = 1:8
        task_name = task_names_run{tsk_idx}
        task_traj_mat_name = sprintf('/core_scripts/data/Sha_pairedProjections_power/Sha_trajs_using_all_%s.mat', task_name);
        load(task_traj_mat_name, 'trajs_all');
        
        for sub_start = 1:set_size:(901-set_size) %% 900 subs compared with template for correlation
            sub_end = sub_start + set_size-1;
            
            max_correlation = -inf;
            
            matched_task = 0;
            
            for i = 1:length(template_data)
                
                trajs_curr = [];
                c = 0;
                for sub = sub_start:sub_end
                    c = c+1;
                    trajs_curr(c).xsm = trajs_all(i).trajs(sub).xsm;
                    
                end
                
                
                template_mean_trajectories = template_data(i).mean_trajectories;
                template_params = template_data(i).estParams;
                
                mean_correlation = find_mean_correlation(template_mean_trajectories, template_params, trajs_curr);
                
                if mean_correlation > max_correlation
                    max_correlation = mean_correlation;
                    matched_task = i;
                end
            end
%                     fprintf('winner : %s\n', task_names_run{matched_task})
            confusion_mat(tsk_idx, matched_task) =  confusion_mat(tsk_idx, matched_task) + 1;
        end
        
        confusion_mat
        
    end
    
    save(sprintf('data/Sha_confusion_mat_set_size_%02d.mat', set_size), 'confusion_mat');
    
end

%% plot

clear;
close all;
make_it_tight = true;
subplot = @(m,n,p) subtightplot (m, n, p, [0 0.05], [0.3 0.15], [0.1 0.02]);
if ~make_it_tight,  clear subplot;  end
task_names_run  = {'rest',   'WM',    'lang',  'motor', 'social', 'gambling', 'relational', 'emotion'};
tot_task = length(task_names_run);

% mainFig = figure('visible','off'); set(gcf,'Position',[10,10,1500, 350]);
c = 1;
for set_size = [1]
    load(sprintf('data/Sha_confusion_mat_set_size_%02d.mat', set_size), 'confusion_mat');
    factor = 100/sum(confusion_mat(1, :));
    confusion_mat = confusion_mat*factor;
    min_acc = min(diag(confusion_mat));
    max_acc = max(diag(confusion_mat));
    mean_acc = mean(diag(confusion_mat));
    fprintf('\nmin : %f, mean : %f, max : %f', min_acc, mean_acc, max_acc);
    imagesc(confusion_mat, [0 100]),colormap(b2r(0, 100)), colorbar;
    axis square;
    for r = 2:size(confusion_mat, 1)
%         text(r-0.4, r, sprintf('%d', round(confusion_mat(r,r))), 'FontSize', 10, 'Color', [0, 0, 0]);
        hold on;
        plot([r-0.5  r-0.5],[0.5 8.5],'-','Color',[0.8 0.8 0.8]); plot([0.5 8.5],[r-0.5  r-0.5],'-','Color',[0.8 0.8 0.8]);
    end
    title(sprintf('\\textbf{GPFA traj based pred}\n\\textbf{min : %.2f, mean : %.2f, max : %.2f}', min_acc, mean_acc, max_acc), 'Interpreter', 'Latex', 'FontSize', 20);
    set(gca, 'XTick', 1:8, 'XTickLabel', task_names_run, 'FontName','Times','fontsize',10,'XTickLabelRotation',45);
    set(gca, 'YTick', 1:8, 'YTickLabel', task_names_run, 'FontName','Times','fontsize',10,'YTickLabelRotation',0);
    set(gca,'TickLength',[0, 0]);
    xlabel('Predicted', 'FontSize', 12);
    if c==1
        ylabel('Actual', 'FontSize', 12);
    end
    
    fprintf('\n');
    c=c+1;
end