%%%%%%%%%%%
% To get spectra information for all tasks, we're taking same number of timepoints available across all tasks : 176
%%%%%%%%%%%

clc;
clear; close all;
addpath(genpath('gpfa_v0203'));
addpath(genpath('chronux_2_12'));

task_names_run  = {'rest',   'WM',    'lang',  'motor', 'social', 'gambling', 'relational', 'emotion'};
task_names_cells = {'resting','task7', 'task2', 'task4', 'task6',  'task3',    'task5',      'task1'}; %matrix names

rmpath(genpath('spm12'));
rmpath(genpath('spm8'));

clc;
Fs=1000/720;

%% create template spectra
for minimize=1
    %% gpfa
    clear spectra;
    figure;
    for task=1:8
        disp(task_names_run{task});
        load(sprintf('/core_scripts/data/Sha_pairedProjections/Sha_extracted_trajs_%s_entire_templates.mat',task_names_run{task}), 'trajs');
        os_cat=[];
        for sub=1:100
            
            X=trajs(sub).xsm;
            T=size(X,2)/Fs;
            data=X(:,1:176)';%****************** data = time x components
            
            spectra(sub,task) = amri_sig_fractal(data, Fs);
            
            os=spectra(sub,task).osci;
            os_cat=cat(3,os_cat,os);
            
        end
        
        ansr=mean(os_cat,3);
        spectra_mean(task).osci=ansr; spectra_mean(task).freq=spectra(1,task).freq;
        
        subplot(2,4,task);
        plot(spectra(1,task).freq, ansr); xlabel('frequency Hz');
        xlim([0 max(spectra(1,task).freq)]);
        title(task_names_run{task})
        
    end
    save('spectra_templates_mean_gpfa_v2.mat','spectra_mean');
    suptitle('GPFA Oscillatory component - avg. for 100 template subs');
    disp('gpfa template spectra ready!');
    
    %% gpfa orth
    clear spectra;
    figure;
    for task=1:8
        disp(task_names_run{task});
        load(sprintf('/core_scripts/data/Sha_pairedProjections/Sha_extracted_trajs_%s_entire_templates_orth.mat',task_names_run{task}), 'trajs');
        os_cat=[];
        for sub=1:100
            
            X=trajs(sub).xorth;
            T=size(X,2)/Fs;
            data=X(:,1:176)';% data = time x components
            
            %  plotting for low dimension data
            spectra(sub,task) = amri_sig_fractal(data, Fs);
            os=spectra(sub,task).osci;
            os_cat=cat(3,os_cat,os);
            
        end
        ansr=mean(os_cat,3);
        spectra_mean(task).osci=ansr; spectra_mean(task).freq=spectra(1,task).freq;
        subplot(2,4,task);
        plot(spectra(1,task).freq, ansr); xlabel('frequency Hz');
        xlim([0 max(spectra(1,task).freq)]);
        title(task_names_run{task})
        
    end
    save('spectra_templates_mean_orth_v2.mat','spectra_mean');
    suptitle('Orth Oscillatory component - avg. for 100 template subs');
    disp('orth template spectra ready!');
    
    %% pca
    clear spectra;
    figure;
    for task=1:8
        disp(task_names_run{task});
        load(sprintf('/core_scripts/data/Sha_pairedProjections_power_pca/Sha_extracted_trajs_%s_entire_templatesubs.mat',task_names_run{task}), 'trajs');
        os_cat=[];
        for sub=1:100
            
            X=trajs(sub).xsm;
            T=size(X,2)/Fs;
            data=X(1:176,:); % data = time x components
            
            %  plotting for low dimension data
            spectra(sub,task) = amri_sig_fractal(data, Fs,'detrend',1);% 'frange', [0 Fs/2]);
            os=spectra(sub,task).osci;
            os_cat=cat(3,os_cat,os);
            
        end
        ansr=mean(os_cat,3);
        spectra_mean(task).osci=ansr; spectra_mean(task).freq=spectra(1,task).freq;
        subplot(2,4,task);
        plot(spectra(1,task).freq, ansr); xlabel('frequency Hz');
        xlim([0 max(spectra(1,task).freq)]);
        title(task_names_run{task})
        
    end
    save('spectra_templates_mean_PCA_v2.mat','spectra_mean');
    
    suptitle('PCA Oscillatory component - avg. for 100 template subs');
    disp('pca template spectra ready!');
end
%% ******* gpfa : with test-task-projected trajectories (all tp), generate spectra and compare against test-task spectra templat
load('data/spectra_templates_mean_gpfa_v2.mat');
load('Fig1/data/template_data_power_templates.mat');

pathh='/core_scripts/data/Sha_pairedProjections';
comps= size(spectra_mean(1).osci,2); %42

clear corr_oscilComp corr_latentSeries;
confusion_mat = zeros(8, 8);
disp('...gpfa start...');

for sub_task=1:8 %actual task
    disp(task_names_run{sub_task});
    clear trajs_all;
    load(sprintf('%s/Sha_entiretrajs_using_all_%s.mat', pathh,task_names_run{sub_task}));
    for sub=1:900
        if mod(sub,50)==1
            fprintf('%d ', sub);
        end
        
        for tsk_idx = 1:8 %projected using task/ task being tested against
            % corr based on oscil. component
            data=trajs_all(tsk_idx).trajs(sub).xsm(:,1:176)'; %* data = time x components
            s = amri_sig_fractal(data, Fs);
            for c=1:comps
                x=s.osci(:,c);
                y=spectra_mean(tsk_idx).osci(:,c);
                corr_oscilComp(sub,sub_task,tsk_idx).corrs(c)=corr(x,y);
            end
            
            % corr based on latent series
            for c=1:comps
                x=trajs_all(tsk_idx).trajs(sub).xsm(c,11:110)';
                y=template_data(tsk_idx).mean_trajectories(c,:)';
                corr_latentSeries(sub,sub_task,tsk_idx).corrs(c)=corr(x,y);
            end
        end
    end
    save('Sha_corr_oscilComp_and_latentSeries_gpfa.mat', 'corr_oscilComp','corr_latentSeries','-v7.3');
end
save('Sha_corr_oscilComp_and_latentSeries_gpfa.mat', 'corr_oscilComp','corr_latentSeries','-v7.3');

disp('gpfa done');

%% ******gpfa Orth : with test-task-projected trajectories (all tp), generate spectra and compare against test-task spectra templat
%     disp('...gpfaOrth start...');
%
% clear spectra_mean;
% load('data/spectra_templates_mean_orth_v2.mat');
% clear template_data;
% load('Fig1/data/template_data_power_templates_with_orth.mat');
%
% pathh='/core_scripts/data/Sha_pairedProjections';
% comps= size(spectra_mean(1).osci,2); %42
% clear corr_oscilComp corr_latentSeries;
% confusion_mat = zeros(8, 8);
%    for sub_task=1:8 %actual task
%        disp(task_names_run{sub_task});
%              clear trajs_all;
%              load(sprintf('%s/Sha_entiretrajs_using_all_%s_orth.mat', pathh,task_names_run{sub_task}));
%         for sub=1:900
%             if mod(sub,50)==1
%             fprintf('%d ', sub);
%             end
%
%               for tsk_idx = 1:8 %projected using task/ task being tested against
%                 % corr based on oscil. component
%                     data=trajs_all(tsk_idx).trajs(sub).xorth(:,1:176)'; %* data = time x components
%                     s = amri_sig_fractal(data, Fs);
%                      for c=1:comps
%                         x=s.osci(:,c);
%                         y=spectra_mean(tsk_idx).osci(:,c);
%                         corr_oscilComp(sub,sub_task,tsk_idx).corrs(c)=corr(x,y);
%                      end
%
%                     % corr based on latent series
%                      for c=1:comps
%                         x=trajs_all(tsk_idx).trajs(sub).xorth(c,11:110)';
%                         y=template_data(tsk_idx).mean_trajectories_orth(c,:)';
%                         corr_latentSeries(sub,sub_task,tsk_idx).corrs(c)=corr(x,y);
%                      end
%              end
%         end
%
%     save('Sha_corr_oscilComp_and_latentSeries_gpfaOrth.mat', 'corr_oscilComp','corr_latentSeries','-v7.3');
% end
%     save('Sha_corr_oscilComp_and_latentSeries_gpfaOrth.mat', 'corr_oscilComp','corr_latentSeries','-v7.3');
%
%
%         disp('orth done');

%% ****** PCA : with test-task-projected trajectories (all tp), generate spectra and compare against test-task spectra templat
disp('...pca start...');

clear spectra_mean;
load('data/spectra_templates_mean_PCA_v2.mat');
clear template_data;
load('Fig1/data/template_data_power_templates_PCA.mat');

pathh='/core_scripts/data/Sha_pairedProjections_power_pca';
comps= size(spectra_mean(1).osci,2); %42

confusion_mat = zeros(8, 8);
clear corr_oscilComp corr_latentSeries;

for sub_task=1:8 %actual task
    disp(task_names_run{sub_task});
    clear trajs_all;
    load(sprintf('%s/Sha_entiretrajs_using_all_%s_pca.mat', pathh,task_names_run{sub_task}));
    for sub=1:900
        if mod(sub,50)==1
            fprintf('%d ', sub);
        end
        clear match_corr;
        
        
        for tsk_idx = 1:8 %projected using task/ task being tested against
            % corr based on oscil. component
            data=trajs_all(tsk_idx).trajs(sub).xsm(1:176,:); %* data = time x components
            s = amri_sig_fractal(data, Fs);
            for c=1:comps
                x=s.osci(:,c);
                y=spectra_mean(tsk_idx).osci(:,c);
                corr_oscilComp(sub,sub_task,tsk_idx).corrs(c)=corr(x,y);
            end
            
            % corr based on latent series
            for c=1:comps
                x=trajs_all(tsk_idx).trajs(sub).xsm(11:110,c);
                y=template_data(tsk_idx).mean_trajectories_PCA(c,:)';
                corr_latentSeries(sub,sub_task,tsk_idx).corrs(c)=corr(x,y);
            end
        end
    end
    
    save('Sha_corr_oscilComp_and_latentSeries_PCA.mat', 'corr_oscilComp','corr_latentSeries','-v7.3');
end
save('Sha_corr_oscilComp_and_latentSeries_PCA.mat', 'corr_oscilComp','corr_latentSeries','-v7.3');

disp('pca done');

