%% use LOWER TRIANGLE Partial Corr connectivity to predict behaviour scores
clear;
addpath(genpath('gpfa_v0203'));
addpath(genpath('chronux_2_12'));
%%
%%%%%%%%%%%%%% changeable params %%%%%%%%%%%%%%%%%%%%
task_names_run  = {'rest',   'WM',    'lang',  'motor', 'social', 'gambling', 'relational', 'emotion'};
display_name_cells={'rest',   'WM',    'lang',  'motor', 'social', 'gambling', 'relational', 'emotion'}; %human name for the task variables
task_names_cells = {'resting','task7', 'task2', 'task4', 'task6',  'task3',    'task5',      'task1'}; %matrix names

metrics=...
    {'MMSE-Score', 'PSQI-Score', 'Endurance-AgeAdj', 'Strength-AgeAdj', 'GaitSpeed-Comp', ...
    'Dexterity-AgeAdj', 'PicSeq-AgeAdj', 'CardSort-AgeAdj', 'Flanker-AgeAdj', 'PMAT24-A-CR', 'ReadEng-AgeAdj', ...
    'PicVocab-AgeAdj', 'ProcSpeed-AgeAdj', 'DDisc-AUC-200', 'DDisc-AUC-40K', 'VSPLOT-TC', 'VSPLOT-OFF', 'SCPT-SEN', ...
    'SCPT-SPEC', 'IWRD-TOT','ListSort-AgeAdj', 'PMAT24-A-SI', 'PMAT24-A-RTCR', 'VSPLOT-CRTE', 'SCPT-TPRT', ...
    'SCPT-LRNR', 'IWRD-RTC'};
for l=1:27
    if l==1
        clc;
    end
    fprintf('%d:%s\n',l,metrics{l}) ;
end
addpath(genpath('Corr_toolbox_v2'));

% Load the behavioral data
BMcsvNAN = importdata('data/behavioralScoresFinalThousandSubjects.csv');
load('Fig1/data/predictOn_subject_IDs.mat')
% check subject overlap : behav. data is available for 896/900 subjects
behav_subs=BMcsvNAN.data(:,2);
exclude_subs=setdiff(predictOn_subject_IDs,behav_subs);%= [668361 ,      685058,      872158,      937160]

find(predictOn_subject_IDs==937160) %(exclude_subs are the last 4 in 900)

% locate the 896 in behav data exel and save their indices in order
% checked: their indices are also in ascending order of sub id only
for s=1:896
    idx(s)=find(behav_subs==predictOn_subject_IDs(s));
end

% subset the behav data for just these indices
BMcsvNAN_896=BMcsvNAN.data(idx,:);

% some scores could be NA/zero, remove such subjects

%1. For some subjects, scores are NA ignore these subjects
use_ind=[1:896];
for i = [3:29]  % only objective scores
    disp(strcat('| Column id: ',num2str(i)));
    
    zeroScoreSubjects = find(isnan(BMcsvNAN_896(:, i))); disp(length(zeroScoreSubjects));
    use_ind = setdiff(use_ind, zeroScoreSubjects);
end

%2. For some subjects, scores are 0 ignore these subjects
for i = [3:23 25:29] %24 is PMAT24_A_SI - zero is legit here
    disp(strcat('| column id: ',num2str(i)));
    zeroScoreSubjects = find(BMcsvNAN_896(:, i)==0); disp(length(zeroScoreSubjects));
    use_ind = setdiff(use_ind, zeroScoreSubjects);
end

%871 subjects remain, with all scores data available, save their sub ids
SubIDs_use_ind=BMcsvNAN_896(use_ind,2);
BM_data=BMcsvNAN_896(use_ind,3:29); % only objective scores

for s=1:length(use_ind)
    test_idx(s)=find(predictOn_subject_IDs == BMcsvNAN_896(use_ind(s),2));
end

%% use , each task's, 42x41/2 partial corr vals, to predict, each score
load('PC_900subsGPFA.mat','PC_900subs_gpfa');%see X4_Calculate_PC.m
no_comp=42;
clear Result;
for task=1:7
    X_subs=PC_900subs_gpfa(task+1).tested(task+1).PC(:,test_idx);
    
    %reshape input to 42x42xno_sub
    all_mats=[];
    for bla=1:size(X_subs,2)
        a=X_subs(:,bla);
        blu=zeros(no_comp,no_comp);
        blu(logical(triu(ones(no_comp,no_comp),1)))=a;blu=blu+blu';
        all_mats=cat(3,all_mats,blu);
    end
    clear Res;
    parfor i = [1:size(BM_data,2)]
        disp(strcat('**/ task: ',task_names_run{task+1},'| Score id: ',num2str(i)));
        tic;
        all_behav = BM_data(:,i);
        %% CPM
        [behav_pred_pos, behav_pred_neg, behav_pred_both, behav_pred_pos_svm, behav_pred_neg_svm, behav_pred_both_svm,...
            behav_pred_all_con_svm,conn_imp_pos,conn_imp_neg,conn_imp_both, conn_imp_pos_weights, conn_imp_neg_weights,...
            conn_imp_both_weights,betas] = predict_behave_10fold(all_mats, all_behav, 'Pearson', 'Normal', 0.05);
        
        toc;
        
        
        Res(i).y_true = all_behav;
        Res(i).betas_both_sum=betas; % added later and saved in _with_betas version
        Res(i).y_pred_pos= behav_pred_pos;
        Res(i).y_pred_neg= behav_pred_neg;
        Res(i).y_pred_both= behav_pred_both;
        Res(i).y_pred_pos_svm= behav_pred_pos_svm;
        Res(i).y_pred_neg_svm= behav_pred_neg_svm;
        Res(i).y_pred_both_svm= behav_pred_both_svm;
        Res(i).y_pred_all_con_svm= behav_pred_all_con_svm;
        
        Res(i).y_conn_imp_pos = conn_imp_pos;
        Res(i).y_conn_imp_neg = conn_imp_neg;
        Res(i).y_conn_imp_both = conn_imp_both;
        
        Res(i).y_conn_imp_pos_weights = conn_imp_pos_weights;
        Res(i).y_conn_imp_neg_weights = conn_imp_neg_weights;
        Res(i).y_conn_imp_both_weights = conn_imp_both_weights;
        %  'y_true','y_pred_pos','y_pred_neg','y_pred_both',...
        %     'y_pred_pos_svm','y_pred_neg_svm','y_pred_both_svm','y_pred_all_con_svm',...
        %     'y_conn_imp_pos','y_conn_imp_neg','y_conn_imp_both',...
        %     'y_conn_imp_pos_weights','y_conn_imp_neg_weights','y_conn_imp_both_weights');
    end
    
    Result(task).task_name=task_names_run{task+1};
    Result(task).Res=Res;
    save('data/WS_Sha_PC_toPred_score_with_betas.mat','Result');
    
end

%% plot
load('data/WS_Sha_PC_toPred_score.mat','Result');
titl='predict score using GPFA partial corr mat significant connections';
against={'y_pred_pos','y_pred_neg','y_pred_both','y_pred_pos_svm','y_pred_neg_svm','y_pred_both_svm'};
descrip={'sum of (+)','sum of (-)','both sums','+ conns','- conns','both conns'}
for a= 1:length(against)
    for t = 1:7
        for s=1:27
            predictions= eval(strcat(' Result(t).Res(s).',against{a}));
            [ rVals(a,t,s), pVals(a,t,s)] = corr(Result(t).Res(s).y_true,predictions);
        end
    end
end

save('data/prediction_accuracy.mat','rVals','pVals')

LR=load('data/prediction_accuracy','rVals');
LRp=load('data/prediction_accuracy','pVals');

for a = 1:length(against)
    [~,~,~,p_LR_BHsig_ind,~]=multiple_comparisons_correction_01...
        (reshape(LRp.pVals(a,:,:),1,(7*27))    ,'Benjaminihochberg');
    
    LR_ind=zeros(length(p_LR_BHsig_ind),2);
    
    for pl = 1:length(p_LR_BHsig_ind)
        [ti,s]=ind2sub([7,27],p_LR_BHsig_ind(pl));
        LR_ind(pl,:)=[ti,s];
    end
    
    [~,idx] = sort(LR_ind(:,1)); % sort just the first column
    sortedLR_ind = LR_ind(idx,:);
    
    p_LR_BHsig_mat = zeros(7,27);
    for i = 1:length(LR_ind)
        p_LR_BHsig_mat(LR_ind(i,1) , LR_ind(i,2)) = 1;
    end
    
    p_LR_BHsig_mats(a).p_LR_BHsig_mat=p_LR_BHsig_mat;
    
end

save('data/CPM_BHsig01','p_LR_BHsig_mats');

%% plot 'both sums', marking p<0.01
a=3;

behavior_scores=[1 2];
motor_scores=[3 4 5 6 ];
cognition_scores=[7 8 9 13 14 15 ];
fluid_intel_scores =[10 22 23];
lang_scores=[11 12];
Spatial_Orientation=[16 17 24];
sustained_attn=[18 19 25 26];
memory =[21 20 27];

all_scores =[behavior_scores lang_scores motor_scores cognition_scores fluid_intel_scores Spatial_Orientation ...
    sustained_attn memory];

load('data/prediction_accuracy','rVals');

figure;
mat=squeeze(rVals(a,:,all_scores));
imagesc(mat); colormap(b2r(-0.2,0.3));hold on;
colorbar; ylabel(descrip{a});
set(gca,'XTick',[1:27]); set(gca,'YTick',[1:7]);  set(gca,'XTickLabels',all_scores);
set(gca,'YTickLabels',{'W','L','M','S','G','R','E'});ax=gca; ax.TickLength = [0.0041, 0.0041];
ax=gca; ax.TickLength = [0.0041, 0.0041];
clear p_LR_BHsig_mats;
load('data/CPM_BHsig01'); %plot 0.01 BH sig in black
for r = 1:27
    for c = 1:7
        if (p_LR_BHsig_mats(a).p_LR_BHsig_mat(c,all_scores(r)))>0
            plot ([r-0.47 r-0.47],[c+0.47 c-0.47],'k','LineWidth',1.5) ;plot ([r+0.47 r+0.47],[c+0.47 c-0.47],'k','LineWidth',1.5) ;hold on;
            plot ([r-0.47 r+0.47],[c+0.47 c+0.47],'k','LineWidth',1.5) ;plot ([r-0.47 r+0.47],[c-0.47 c-0.47],'k','LineWidth',1.5) ;hold on;
        end
    end
    
end


set(gcf,'Position',[  520   587   929   211]);

