function [behav_pred_pos, behav_pred_neg, behav_pred_both, behav_pred_pos_svm, behav_pred_neg_svm, behav_pred_both_svm,...
    behav_pred_all_con_svm,conn_imp_pos,conn_imp_neg,conn_imp_both, conn_imp_pos_weights, conn_imp_neg_weights,...
    conn_imp_both_weights,betas] = predict_behave_10fold(all_mats, all_behav, cor, thresh_corr, t, age_all)

% function [R_pos, R_neg, R_all] = predict_behave(all_mats, all_behav, cor, threshold)
%% MODIFIED
% threshold for feature selection
thresh = t;
%---------------------------------------------------

no_sub = size(all_mats, 3);
no_node = size(all_mats, 1);

behav_pred_pos = zeros(no_sub, 1);
behav_pred_neg = zeros(no_sub, 1);
behav_pred_both = zeros(no_sub, 1);
behav_pred_pos_svm = zeros(no_sub, 1);
behav_pred_neg_svm = zeros(no_sub, 1);
behav_pred_both_svm = zeros(no_sub, 1);
behav_pred_all_con_svm = zeros(no_sub, 1);
corr_val_pos = zeros(no_sub, 1);
corr_val_neg = zeros(no_sub, 1);
conn_imp_pos = zeros(size(all_mats,1),size(all_mats,1));
conn_imp_neg = zeros(size(all_mats,1),size(all_mats,1));
conn_imp_both = zeros(size(all_mats,1),size(all_mats,1));

sz=floor(no_sub/10);
for k=1:9
leave_inds(k).inds=[(sz*(k-1)+1) : (sz*k)];
end
leave_inds(10).inds=[(sz*9)+1 : no_sub];

for kfold=1:10
    %     fprintf('\n Leaving out subject # %d', leftout);
    
    % leave out subject from matrix and behavior
    train_subs=setdiff(1:no_sub,leave_inds(kfold).inds);
    
    train_vcts =  reshape(all_mats(:,:,train_subs), [], length(train_subs));
    train_behav = all_behav(train_subs);
    
%     train_age = age_all;
%     train_age(leftout) = [];
    
    % correlate all edges with behavior
    if strcmp(cor,'Pearson')
        %---------------------------Pearson-------------------------
        [r_mat, p_mat] = corr(train_vcts', train_behav(:));
        %         R, P = r_mat, p_mat;
        r_mat = reshape(r_mat, no_node, no_node);
        p_mat = reshape(p_mat, no_node, no_node);
    elseif strcmp(cor,'Spearman')
        %--------------------------Spearman-------------------------
        [r_mat, p_mat] = corr(train_vcts', train_behav, 'type', 'Spearman');
        r_mat = reshape(r_mat, no_node, no_node);
        p_mat = reshape(p_mat, no_node, no_node);
    elseif strcmp(cor,'Partial')
        %----------------------Partial with age---------------------
        [r_mat, p_mat] = partialcorr(train_vcts', train_behav, train_age);
        r_mat = reshape(r_mat, no_node, no_node);
        p_mat = reshape(p_mat, no_node, no_node);
    else
        %--------------------------Robust Regression----------------
        edge_no = size(train_vcts, 1);
        r_mat = zeros(1, edge_no);
        p_mat = zeros(1, edge_no);
        
        for edge_i = 1:edge_no
            [~, stats] = robustfit(train_vcts(edge_i,:)', train_behav);
            cur_t = stats.t(2);
            r_mat(edge_i) = sign(cur_t)*sqrt(cur_t^2/no_sub-1-2+cur_t^2);
            p_mat(edge_i) = 2*tcdf(cur_t, no_sub-1-2);     % two-tailed
        end
        
        r_mat = reshape(r_mat, no_node, no_node);
        p_mat = reshape(p_mat, no_node, no_node);
    end
    %------------------------Correlation Calculated------------------
    
    
    % set threshold and define masks
    pos_mask = zeros(no_node, no_node);
    neg_mask = zeros(no_node, no_node);
    
    if strcmp(thresh_corr, 'Normal')
        %--------------------------Using threshold---------------------------
        pos_edges = find(r_mat > 0 & p_mat <= thresh);
        neg_edges = find(r_mat < 0 & p_mat <= thresh);
        both_edges = union(pos_edges, neg_edges);
        
        pos_mask(pos_edges) = 1;
        neg_mask(neg_edges) = 1;
        %--------------------------Using threshold---------------------------
    else
        %---------------------------Sigmoid-weighting------------------------
        pos_edges = find(r_mat > 0);
        neg_edges = find(r_mat < 0);
        both_edges = union(pos_edges, neg_edges);
        %convert p threshold to r threshold
        T = tinv(thresh/2, no_sub-1-2);
        R = sqrt(T^2/(no_sub-1-2+T^2));
        
        % create weight mask using sigmoidal function
        % weight = 0.5, when correlation = R/3
        % weight = 0.88, when correlation = R
        pos_mask(pos_edges) = sigmf(r_mat(pos_edges), [3/R, R/3]);
        neg_mask(neg_edges) = sigmf(r_mat(neg_edges), [-3/R, R/3]);
        %---------------------------Sigmoid-weighting------------------------
    end
    
%     all_mask = tril(ones(size(all_mats, 1)), -1);
    pos_mask = tril(pos_mask, -1);
    neg_mask = tril(neg_mask, -1);
    pos_edges = find(pos_mask == 1);
    neg_edges = find(neg_mask == 1);
    both_edges = union(pos_edges, neg_edges);
    conn_imp_pos = conn_imp_pos + pos_mask;
    conn_imp_neg = conn_imp_neg + neg_mask;
    both_mask = pos_mask | neg_mask;             % pos + neg
    conn_imp_both = conn_imp_both + both_mask;
    
%     all_con = find(tril(ones(size(all_mats, 1), size(all_mats, 1)), -1) == 1);   %all connections
    
    %get sum of all edges in Train subs (divided by 2 to control for the fact tht matrices are symmetric)
    train_sumpos = zeros(length(train_subs), 1);
    train_sumneg = zeros(length(train_subs), 1);
    
    ptr=1;
    for ss = train_subs
        train_sumpos(ptr) = sum(sum(all_mats(:,:,ss).*pos_mask));
        train_sumneg(ptr) = sum(sum(all_mats(:,:,ss).*neg_mask));
        ptr=ptr+1;
    end
    
    %     train_sumpos = train_sumpos / max(train_sumpos);
    %     train_sumpos = sigmf(train_sumpos, [2, 1]);
    %     train_sumneg = train_sumneg / max(train_sumneg);
    %     train_sumneg = sigmf(train_sumneg, [2, 1]);
    
    corr_val_pos(kfold,1) = corr(train_sumpos, train_behav);
    corr_val_neg(kfold,1) = corr(train_sumneg, train_behav);
    %     train_sumpos = abs(train_sumpos);
    %     train_sumneg = abs(train_sumneg);
    %     train_sumpos = zscore(train_sumpos);
    %     train_sumneg = zscore(train_sumneg);
    
    pos_mat = zeros(length(train_subs), size(pos_edges,1));
    neg_mat = zeros(length(train_subs), size(neg_edges,1));
    both_mat = zeros(length(train_subs), size(both_edges,1));
%     all_con_mat = zeros(no_sub-1, size(all_con,1));
ptr=1;
    for sub = train_subs
        a = all_mats(:,:,sub);
        pos_mat(ptr, :) = (a(pos_edges));
        neg_mat(ptr, :) = (a(neg_edges));
        both_mat(ptr, :) = (a(both_edges));
%         all_con_mat(ptr, :) = a(all_con);
        ptr=ptr+1;
    end
    %     pos_mat = zscore(pos_mat);
    %     neg_mat = zscore(neg_mat);
    % build model on Train set
    %----------------------------Poly-fit--------------------------------
    %     [fit_pos, S_pos, MU_pos] = polyfit(train_sumpos, train_behav, 1);
    %     [fit_neg, S_neg, MU_neg] = polyfit(train_sumneg, train_behav, 1);
    fit_pos = polyfit(train_sumpos, train_behav, 1);
    fit_neg = polyfit(train_sumneg, train_behav, 1);
    %     f = fittype('poly1');
    %     fit1 = fit(train_sumpos, train_behav, f);
    %     fdata = feval(fit1, train_sumpos);
    %     I = abs(fdata - train_behav) > 1.5 * std(train_behav);
    %     outliers = excludedata(train_sumpos, train_behav, 'indices', I);
    %     fit2 = fit(train_sumpos, train_behav, f, 'Exclude', outliers);
    %     fit_pos = fitlm(train_sumpos, train_behav, 'linear', 'RobustOpts', 'on');
    %     fit_neg = fitlm(train_sumneg, train_behav, 'linear', 'RobustOpts', 'on');
    %--------------------Combining Pos and Neg features------------------
    %     b = regress(train_behav, [train_sumpos, train_sumneg, ones(no_sub-1,1)]);
    b = fitglm([train_sumpos, train_sumneg], train_behav, 'linear');
    betas(kfold).coeff=b.Coefficients;
    betas(kfold).pos_conns=mean(all_mats(:,:,train_subs),3).*pos_mask;
    betas(kfold).neg_conns=mean(all_mats(:,:,train_subs),3).*neg_mask;
    betas(kfold).pos_mask=pos_mask;
    betas(kfold).neg_mask=neg_mask;
    %% SVR Model
    mdl_pos = fitrlinear(pos_mat, train_behav, 'Learner','svm','Solver','sgd','Regularization','ridge', 'LearnRate',0.1); 
    mdl_neg = fitrlinear(neg_mat, train_behav, 'Learner','svm','Solver','sgd','Regularization','ridge', 'LearnRate',0.1); 
    mdl_both = fitrlinear(both_mat, train_behav, 'Learner','svm','Solver','sgd','Regularization','ridge', 'LearnRate',0.1); 
%     mdl_all_con = fitrlinear(all_con_mat, train_behav);
    % End
    current_pos_weights = zeros(size(all_mats,1),size(all_mats,1));
    current_pos_weights(pos_edges) = mdl_pos.Beta;
    
    current_neg_weights = zeros(size(all_mats,1),size(all_mats,1));
    current_neg_weights(neg_edges) = mdl_neg.Beta;
    
    current_both_weights = zeros(size(all_mats,1),size(all_mats,1));
    current_both_weights(both_edges) = mdl_both.Beta;
    
    conn_imp_pos_weights(kfold).current_pos_weights = current_pos_weights;
    conn_imp_neg_weights(kfold).current_neg_weights = current_neg_weights;
    conn_imp_both_weights(kfold).current_both_weights = current_both_weights;
    
    % run model on Test sub
    for lo = leave_inds(kfold).inds
    test_mat = all_mats(:,:,lo);
    test_pos = test_mat(pos_edges);
    test_neg = test_mat(neg_edges);
    test_both = test_mat(both_edges);
%     test_all_con = test_mat(all_con);
    %     test_mat_pos = (test_mat - MU_pos(1))/MU_pos(2);
    %     test_mat_neg = (test_mat - MU_neg(1))/MU_neg(2);
    test_sumpos = sum(sum(test_mat.*pos_mask));
    %     test_sumpos = test_sumpos / max(train_sumpos);
    %     test_sumpos = sigmf(test_sumpos, [2, 1]);
    
    test_sumneg = sum(sum(test_mat.*neg_mask));
    %     test_sumneg = test_sumneg / max(test_sumneg);
    %     test_sumneg = sigmf(test_sumneg, [2, 1]);
    
    behav_pred_pos(lo) = polyval(fit_pos, (test_sumpos));  %fit_pos(1)*test_sumpos + fit_pos(2);
    behav_pred_neg(lo) = polyval(fit_neg, (test_sumneg));   %fit_neg(1)*test_sumneg + fit_neg(2);
    %     behav_pred_pos(leftout) = predict(fit_pos, test_sumpos);
    %     behav_pred_neg(leftout) = predict(fit_neg, test_sumneg);
    %     behav_pred(leftout) = b(1)*test_sumpos + b(2)*test_sumneg + b(3);
    behav_pred_both(lo) =  b.Coefficients{2,1}*test_sumpos + b.Coefficients{3,1}*test_sumneg + b.Coefficients{1,1};
    
    behav_pred_pos_svm(lo) = predict(mdl_pos, test_pos');
    behav_pred_neg_svm(lo) = predict(mdl_neg, (test_neg'));
    behav_pred_both_svm(lo) = predict(mdl_both, (test_both'));
%     behav_pred_all_con_svm(lo) = predict(mdl_all_con, (test_all_con'));
    end
end
% conn_imp_pos_weights = conn_imp_pos_weights/size(all_mats,3);
% conn_imp_neg_weights = conn_imp_neg_weights/size(all_mats,3);
% conn_imp_both_weights = conn_imp_both_weights/size(all_mats,3);

% compare predicted and observed scores

% [R_pos, P_pos] = predictive_robust(all_behav, behav_pred_pos);

% [R_pos, P_pos] = corr(behav_pred_pos, all_behav, 'type', 'Pearson');
% [R_neg, P_neg] = corr(behav_pred_neg, all_behav, 'type', 'Pearson');
% [R_all, P_all] = corr(behav_pred, all_behav, 'type', 'Pearson');

% figure(1); plot(behav_pred_pos, all_behav, 'r.');lsline
% figure(2); plot(behav_pred_neg, all_behav, 'b.');lsline
% figure(3); plot(behav_pred, all_behav, 'g.');lsline
end