%findLAB Shirer(willard) 90 parcelled data for rl runs of specified subjects
% same was done for Power 264 ROI series as well
%% dependencies
addpath('spm12');
%% core
output_dir = '/output_dir';
%% give subject_IDs here
% load('/core_scripts/data/predictOn_subject_IDs'); subj_ids = predictOn_subject_IDs;
load('/core_scripts/data/template_subjects_IDs'); subj_ids = template_subjects_IDs;

%==========================================
%==========================================

%folder of roi files
roi_main_dir = '...';
roi_names_all = dir(fullfile(roi_main_dir,'*.nii'));
roi_names_sorted = sort({roi_names_all.name});

region_labels = roi_names_sorted;

%hard init with MNI dimensions
number_nodes = length(roi_names_sorted);
labels_cortex=zeros(91,109,91);




for curr_file = 1:number_nodes
    
    roi_file_hdr = spm_vol(fullfile(roi_main_dir,roi_names_sorted{curr_file}));
    
    curr_roi_vol = spm_read_vols(roi_file_hdr);
    
    voxel_indices = find(curr_roi_vol>0);
    
    
    [x,y,z] = ind2sub(size(labels_cortex),voxel_indices);
    
    roi_centers(curr_file,:) = [mean(x),mean(y),mean(z)];
    
    %iterate over all voxels in this net
    for curr_voxel_index = 1:length(voxel_indices)
        
        vox_coords = [x(curr_voxel_index) y(curr_voxel_index) z(curr_voxel_index)];
        
        curr_voxel = voxel_indices(curr_voxel_index);
        
        if labels_cortex(curr_voxel) > 0 %overlapping with already seen net
            
            prev_dist = norm(roi_centers(labels_cortex(curr_voxel),:) - vox_coords);
            
            curr_dist = norm(roi_centers(curr_file,:) - vox_coords);
            
            if curr_dist<prev_dist %if this mean is closer then replace index
                labels_cortex(curr_voxel) = curr_file;
            end
            
            
        else %no overlap with another netwoeks
            
            labels_cortex(curr_voxel) = curr_file;
            
        end
        
    end
    
    
    
    %removing the nii part from file name
    region_labels{curr_file} = region_labels{curr_file}(1:end-4);
end

%==========================================
%==========================================

labels_cortex=labels_cortex(:);
labels_willard = labels_cortex;

for subj=1:length(subj_ids)
    
    
    sub_id = subj_ids(subj);
    
    %clear all nii and nii.gz files before starting
    system('rm -f /output_dir/*.nii');
    system('rm -f /output_dir/*.nii.gz');
    
    
    
    try
        
        %hack to make aws work from matlab
        %before that let's save the env so that it can be set again for fsl
        original_env = getenv('LD_LIBRARY_PATH');
        setenv('LD_LIBRARY_PATH', '');
        
        
        %download data
        system(strcat('aws s3 cp s3://hcp-openaccess/HCP_900/',......
            num2str(sub_id),'/MNINonLinear/Results/rfMRI_REST1_LR/rfMRI_REST1_LR.nii.gz /output_dir/rest.nii.gz'));
        
        system(strcat('aws s3 cp s3://hcp-openaccess/HCP_900/',......
            num2str(sub_id),'/MNINonLinear/Results/tfMRI_EMOTION_LR/tfMRI_EMOTION_LR.nii.gz /output_dir/task1.nii.gz'));
        
        system(strcat('aws s3 cp s3://hcp-openaccess/HCP_900/',......
            num2str(sub_id),'/MNINonLinear/Results/tfMRI_LANGUAGE_LR/tfMRI_LANGUAGE_LR.nii.gz /output_dir/task2.nii.gz'));
        
        system(strcat('aws s3 cp s3://hcp-openaccess/HCP_900/',......
            num2str(sub_id),'/MNINonLinear/Results/tfMRI_GAMBLING_LR/tfMRI_GAMBLING_LR.nii.gz /output_dir/task3.nii.gz'));
        
        system(strcat('aws s3 cp s3://hcp-openaccess/HCP_900/',......
            num2str(sub_id),'/MNINonLinear/Results/tfMRI_MOTOR_LR/tfMRI_MOTOR_LR.nii.gz /output_dir/task4.nii.gz'));
        
        system(strcat('aws s3 cp s3://hcp-openaccess/HCP_900/',......
            num2str(sub_id),'/MNINonLinear/Results/tfMRI_RELATIONAL_LR/tfMRI_RELATIONAL_LR.nii.gz /output_dir/task5.nii.gz'));
        
        system(strcat('aws s3 cp s3://hcp-openaccess/HCP_900/',......
            num2str(sub_id),'/MNINonLinear/Results/tfMRI_SOCIAL_LR/tfMRI_SOCIAL_LR.nii.gz /output_dir/task6.nii.gz'));
        
        system(strcat('aws s3 cp s3://hcp-openaccess/HCP_900/',......
            num2str(sub_id),'/MNINonLinear/Results/tfMRI_WM_LR/tfMRI_WM_LR.nii.gz /output_dir/task7.nii.gz'));
        
        
        %rest scans are huge. Let's use fsl and truncate them to 405 points
        setenv('LD_LIBRARY_PATH', original_env);
        system('fslroi /output_dir/rest.nii.gz /output_dir/rest.nii.gz 0 405');
        
        %un gz the downloaded files using gunzip
        fprintf('Gunzipping\n');
        
        system('gunzip /output_dir/rest.nii.gz');
        system('gunzip /output_dir/task1.nii.gz');
        system('gunzip /output_dir/task2.nii.gz');
        system('gunzip /output_dir/task3.nii.gz');
        system('gunzip /output_dir/task4.nii.gz');
        system('gunzip /output_dir/task5.nii.gz');
        system('gunzip /output_dir/task6.nii.gz');
        system('gunzip /output_dir/task7.nii.gz');
        
        fprintf('roi extraction\n');
        
        %load the files using spm
        bold_rest = spm_read_vols(spm_vol('/output_dir/rest.nii'));
        dims = size(bold_rest);
        bold_rest = reshape(bold_rest,dims(1)*dims(2)*dims(3),dims(4));
        
        tc_rest_willard = zeros(size(bold_rest,2),90);
        
        for i=1:90
            tc_rest_willard(:,i) = (mean(bold_rest(find(labels_willard== i),:)))';
        end
        clear bold_rest;
        
        bold_task1 = spm_read_vols(spm_vol('/output_dir/task1.nii'));
        dims = size(bold_task1);
        bold_task1 = reshape(bold_task1,dims(1)*dims(2)*dims(3),dims(4));
        
        tc_task1_willard = zeros(size(bold_task1,2),90);
        
        for i=1:90
            tc_task1_willard(:,i) = (mean(bold_task1(find(labels_willard== i),:)))';
        end
        clear bold_task1;
        
        bold_task2 = spm_read_vols(spm_vol('/output_dir/task2.nii'));
        dims = size(bold_task2);
        bold_task2 = reshape(bold_task2,dims(1)*dims(2)*dims(3),dims(4));
        
        tc_task2_willard = zeros(size(bold_task2,2),90);
        
        for i=1:90
            tc_task2_willard(:,i) = (mean(bold_task2(find(labels_willard== i),:)))';
        end
        clear bold_task2;
        
        
        bold_task3 = spm_read_vols(spm_vol('/output_dir/task3.nii'));
        dims = size(bold_task3);
        bold_task3 = reshape(bold_task3,dims(1)*dims(2)*dims(3),dims(4));
        
        tc_task3_willard = zeros(size(bold_task3,2),90);
        
        for i=1:90
            tc_task3_willard(:,i) = (mean(bold_task3(find(labels_willard== i),:)))';
        end
        clear bold_task3;
        
        
        
        bold_task4 = spm_read_vols(spm_vol('/output_dir/task4.nii'));
        dims = size(bold_task4);
        bold_task4 = reshape(bold_task4,dims(1)*dims(2)*dims(3),dims(4));
        
        tc_task4_willard = zeros(size(bold_task4,2),90);
        
        for i=1:90
            tc_task4_willard(:,i) = (mean(bold_task4(find(labels_willard== i),:)))';
        end
        clear bold_task4;
        
        
        
        
        bold_task5 = spm_read_vols(spm_vol('/output_dir/task5.nii'));
        dims = size(bold_task5);
        bold_task5 = reshape(bold_task5,dims(1)*dims(2)*dims(3),dims(4));
        
        tc_task5_willard = zeros(size(bold_task5,2),90);
        
        for i=1:90
            tc_task5_willard(:,i) = (mean(bold_task5(find(labels_willard== i),:)))';
        end
        clear bold_task5;
        
        
        
        
        bold_task6 = spm_read_vols(spm_vol('/output_dir/task6.nii'));
        dims = size(bold_task6);
        bold_task6 = reshape(bold_task6,dims(1)*dims(2)*dims(3),dims(4));
        
        tc_task6_willard = zeros(size(bold_task6,2),90);
        
        for i=1:90
            tc_task6_willard(:,i) = (mean(bold_task6(find(labels_willard== i),:)))';
        end
        clear bold_task6;
        
        
        
        bold_task7 = spm_read_vols(spm_vol('/output_dir/task7.nii'));
        dims = size(bold_task7);
        bold_task7 = reshape(bold_task7,dims(1)*dims(2)*dims(3),dims(4));
        
        tc_task7_willard = zeros(size(bold_task7,2),90);
        
        for i=1:90
            tc_task7_willard(:,i) = (mean(bold_task7(find(labels_willard== i),:)))';
        end
        clear bold_task7;
               
    catch e
        fprintf('\n\n%s\n\n',getReport(e));
        fprintf('Some Excption. Ignoring subject... \n');
        setenv('LD_LIBRARY_PATH', original_env);
        clear tc_task1_willard tc_task2_willard tc_task3_willard tc_task4_willard  tc_task5_willard tc_task6_willard tc_task7_willard tc_rest_willard;
        continue;
    end
    
    
    save(strcat(output_dir,num2str(subj),'.mat')...
        ,'tc_rest_willard','tc_task1_willard','tc_task2_willard','tc_task4_willard','tc_task3_willard','tc_task5_willard','tc_task6_willard','tc_task7_willard','region_labels');
    clear tc_task1_willard tc_task2_willard tc_task3_willard tc_task4_willard tc_task5_willard tc_task6_willard tc_task7_willard tc_rest_willard;
    
end
